@@ -1,20 +1,19 @@
-"""Module that pre-processes the notebook for export to HTML.
-"""
+"""Module that pre-processes the notebook for export to HTML."""
 
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
 
-import os
-import io
 import hashlib
-import nbconvert.resources
+import os
+
+from jupyterlab_pygments import JupyterStyle  # type:ignore[import-untyped]
+from pygments.style import Style
+from traitlets import Type, Unicode, Union
 
-from traitlets import Unicode
 from .base import Preprocessor
 
-
 try:
-    from notebook import DEFAULT_STATIC_FILES_PATH
+    from notebook import DEFAULT_STATIC_FILES_PATH  # type:ignore[import-not-found]
 except ImportError:
     DEFAULT_STATIC_FILES_PATH = None
 
@@ -24,15 +23,17 @@ class CSSHTMLHeaderPreprocessor(Preprocessor):
     Preprocessor used to pre-process notebook for HTML output.  Adds IPython notebook
     front-end CSS and Pygments CSS to HTML output.
     """
-    highlight_class = Unicode('.highlight',
-                              help="CSS highlight class identifier"
-    ).tag(config=True)
 
-    style = Unicode('default',
-            help='Name of the pygments style to use'
+    highlight_class = Unicode(".highlight", help="CSS highlight class identifier").tag(config=True)
+
+    style = Union(
+        [Unicode("default"), Type(klass=Style)],
+        help="Name of the pygments style to use",
+        default_value=JupyterStyle,
     ).tag(config=True)
 
     def __init__(self, *pargs, **kwargs):
+        """Initialize the preprocessor."""
         Preprocessor.__init__(self, *pargs, **kwargs)
         self._default_css_hash = None
 
@@ -40,9 +41,9 @@ class CSSHTMLHeaderPreprocessor(Preprocessor):
         """Fetch and add CSS to the resource dictionary
 
         Fetch CSS from IPython and Pygments to add at the beginning
-        of the html files.  Add this css in resources in the 
+        of the html files.  Add this css in resources in the
         "inlining.css" key
-        
+
         Parameters
         ----------
         nb : NotebookNode
@@ -51,29 +52,19 @@ class CSSHTMLHeaderPreprocessor(Preprocessor):
             Additional resources used in the conversion process.  Allows
             preprocessors to pass variables into the Jinja engine.
         """
-        resources['inlining'] = {}
-        resources['inlining']['css'] = self._generate_header(resources)
+        resources["inlining"] = {}
+        resources["inlining"]["css"] = self._generate_header(resources)
         return nb, resources
 
     def _generate_header(self, resources):
-        """ 
-        Fills self.header with lines of CSS extracted from IPython 
+        """
+        Fills self.header with lines of CSS extracted from IPython
         and Pygments.
         """
         from pygments.formatters import HtmlFormatter
-        header = []
-        
-        # Construct path to Jupyter CSS
-        sheet_filename = os.path.join(
-            os.path.dirname(nbconvert.resources.__file__),
-            'style.min.css',
-        )
-        
-        # Load style CSS file.
-        with io.open(sheet_filename, encoding='utf-8') as f:
-            header.append(f.read())
 
-        # Add pygments CSS
+        header = []
+
         formatter = HtmlFormatter(style=self.style)
         pygments_css = formatter.get_style_defs(self.highlight_class)
         header.append(pygments_css)
@@ -81,19 +72,21 @@ class CSSHTMLHeaderPreprocessor(Preprocessor):
         # Load the user's custom CSS and IPython's default custom CSS.  If they
         # differ, assume the user has made modifications to his/her custom CSS
         # and that we should inline it in the nbconvert output.
-        config_dir = resources['config_dir']
-        custom_css_filename = os.path.join(config_dir, 'custom', 'custom.css')
+        config_dir = resources["config_dir"]
+        custom_css_filename = os.path.join(config_dir, "custom", "custom.css")
         if os.path.isfile(custom_css_filename):
             if DEFAULT_STATIC_FILES_PATH and self._default_css_hash is None:
-                self._default_css_hash = self._hash(os.path.join(DEFAULT_STATIC_FILES_PATH, 'custom', 'custom.css'))
+                self._default_css_hash = self._hash(
+                    os.path.join(DEFAULT_STATIC_FILES_PATH, "custom", "custom.css")
+                )
             if self._hash(custom_css_filename) != self._default_css_hash:
-                with io.open(custom_css_filename, encoding='utf-8') as f:
+                with open(custom_css_filename, encoding="utf-8") as f:
                     header.append(f.read())
         return header
 
     def _hash(self, filename):
         """Compute the hash of a file."""
-        md5 = hashlib.md5()
-        with open(filename, 'rb') as f:
+        md5 = hashlib.md5()  # noqa: S324
+        with open(filename, "rb") as f:
             md5.update(f.read())
         return md5.digest()
